# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
# Reads COMMA-DELIMITED CSV file,
# turns it into HTML 'spectrogram'.
# Each line in the CSV becomes a row in HTML.
# The leftmost item in each row is assumed to be the title for the row.
# All items after the title must be numbers.
# We read the numbers and determine max and min.
# We then proportion each number to a color, using 256 shades.
# You get four choices of color scheme: all red, all blue, all green,
# and alternating.
# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =


import os
import glob
import string
from Tkinter import *
import tkFileDialog

# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =

#COLORS:
#Generate three arrays. 256 reds, 256 greens, 256 blues.
Reds=[]
Blues=[]
Greens=[]
for n in range(256):
    rstring='#%.2x0000' % n
    gstring='#00%.2x00' % n
    bstring='#0000%.2x' % n
    Reds.append(rstring)
    Greens.append(gstring)
    Blues.append(bstring)

infilename=''
outfilename=''

ColorType=''
# use this for 'RED', 'GREEN', 'BLUE', 'ALL'

# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =


def PreRead(infilename):
    infp=open(infilename,'rt')
    MaxNum=-10000000.0
    MinNum= 10000000.0
    RowCount=0
    # Read a row in infile
    while 1:
        rowstring=infp.readline()
        if not rowstring: break
        if rowstring=='': break
        # Break the instring, assuming comma-delimited.
        parts=string.split(string.strip(rowstring), ',' )
        if not parts: break
        if len(parts)<2: break
        # do max-push with rowcount
        if len(parts) > RowCount: RowCount = len(parts)
        # Assume first item is row title, not to be treated as colors.
        rowtitle=parts[0]
        for i in range(1,len(parts)):
            try:
                Num = float(string.strip(parts[i]))
            except:
                pass
            if   Num < MinNum: MinNum = Num
            elif Num > MaxNum: MaxNum = Num
        #end parts in row
    # end while
    infp.close()
    return (MinNum,MaxNum,RowCount)

# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =

def ProcessFile(infilename,outfilename,ColorMode):


    # Call routine to read infile, pin down max and min and rowcount
    (MinNum,MaxNum,RowCount) = PreRead(infilename)

    infp=open(infilename,'rt')
    outfp=open(outfilename,'wt')
    # Write header for table
    outfp.write('<html>\n<table border="0">\n')

    # Use Alternator to control colors by mode
    if   ColorMode == 'RED':   Alternator=0
    elif ColorMode == 'GREEN': Alternator=1
    elif ColorMode == 'BLUE':  Alternator=2
    elif ColorMode == 'ALL':   Alternator=0

    while 1:
        # Read a row in infile
        rowstring=infp.readline()
        if not rowstring: break
        if rowstring=='': break

        # Start a table row
        outfp.write('<tr>\n')
        # Break the instring, assuming comma-delimited.
        parts=string.split(string.strip(rowstring), ',' )
        # Assume first item is row title, not to be treated as colors.
        if not parts: break
        if len(parts)<2: break
        rowtitle=parts[0]
        # write a first <td> for the title
        outfp.write('<td>%s</td>' % rowtitle)
        # write the rest of the items as colors.
        for i in range(1,RowCount):
            # Always run max rows to keep table 'squared' even when this row isn't complete
            Num=0.0
            try:
                Num = float(parts[i])
            except:
                Num=0.0
            # Interpolate the number to a 256-base index for colors
            PropNum = int(255.0 * (Num-MinNum)/(MaxNum-MinNum))
            if Num >= 1.0: FillString='%5.1f' % Num
            else: FillString='%0.3f' % Num
            A3=Alternator % 3
            if   (A3) == 0:
                outfp.write('<td bgcolor="%s">%s</td>' % (Reds[PropNum],FillString))
            elif (A3) == 1:
                outfp.write('<td bgcolor="%s">%s</td>' % (Greens[PropNum],FillString))
            elif (A3) == 2:
                outfp.write('<td bgcolor="%s">%s</td>' % (Blues[PropNum],FillString))
        #end parts in row

        # Use Alternator to set next color
        if ColorMode == 'RED':     Alternator=0
        elif ColorMode == 'GREEN': Alternator=1
        elif ColorMode == 'BLUE':  Alternator=2
        elif ColorMode == 'ALL':   Alternator=Alternator+1

        outfp.write('</tr>\n') # end row with </tr> and newline

    # end while

    outfp.write('\n</table>\n</html>') # footer crap

    infp.close()

    outfp.close()

    app.ShowStatus("Done")
# end Process


# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        self.master = master
        master.title("CSV to spectrogram")


        self.buttonGet = Button(self.master, text="Get CSV", command=self.HandleGet)
        self.buttonGet.grid(row=0, column=0, pady=6)

        self.buttonSave = Button(self.master, text="Name output", command=self.HandleSave)
        self.buttonSave.grid(row=0, column=2, pady=6)

        self.StatusEntry = Entry(self.master,width=60)
        self.StatusEntry.grid(row=1,column=0,columnspan=4, pady=6)

        self.buttonR = Button(self.master, text="Go red", command=self.HandleGoRed)
        self.buttonR.grid(row=2, column=0, pady=6)
        self.buttonR.config(state='disabled')

        self.buttonG = Button(self.master, text="Go green", command=self.HandleGoGreen)
        self.buttonG.grid(row=2, column=1, pady=6)
        self.buttonG.config(state='disabled')

        self.buttonB = Button(self.master, text="Go blue", command=self.HandleGoBlue)
        self.buttonB.grid(row=2, column=2, pady=6)
        self.buttonB.config(state='disabled')

        self.buttonA = Button(self.master, text="Go alternating", command=self.HandleGoAll)
        self.buttonA.grid(row=2, column=3, pady=6)
        self.buttonA.config(state='disabled')

        self.master.protocol('WM_DELETE_WINDOW', self.HandleCancel)


    # - - - - - - - - - - - - - - - - - -

    def ShowStatus(self,S): # Just saving repetition
        self.StatusEntry.delete(0,END)
        self.StatusEntry.insert(0,S)
        self.StatusEntry.update_idletasks()

    # - - - - - - - - - - - - - - - - - -

    def HandleGet(self):
        global infilename
        # Pop up file box to get name.
        infilename=tkFileDialog.askopenfilename(defaultextension='.csv',filetypes = [('CSV files', '.CSV')] )

        if not infilename:
            self.ShowStatus("No file selected. Try again.")
            return
        else:
            self.ShowStatus("Now give a name for output.")


    # - - - - - - - - - - - - - - - - - -

    def HandleSave(self):
        global outfilename
        # Pop up file box to get name.
        outfilename=tkFileDialog.asksaveasfilename()
        if not string.count(string.upper(outfilename),'.HTML'):
            base,ext=os.path.splitext(outfilename)
            outfilename=base+'.html'
        self.ShowStatus("Now pick color")
        app.buttonR.config(state='normal')
        app.buttonG.config(state='normal')
        app.buttonB.config(state='normal')
        app.buttonA.config(state='normal')

    # - - - - - - - - - - - - - - - - - -

    def HandleGoRed(self):
        ProcessFile(infilename,outfilename,'RED')

    # - - - - - - - - - - - - - - - - - -

    def HandleGoGreen(self):
        ProcessFile(infilename,outfilename,'GREEN')

    # - - - - - - - - - - - - - - - - - -

    def HandleGoBlue(self):
        ProcessFile(infilename,outfilename,'BLUE')

    # - - - - - - - - - - - - - - - - - -

    def HandleGoAll(self):
        ProcessFile(infilename,outfilename,'ALL')

    # - - - - - - - - - - - - - - - - - -

    def HandleCancel(self):
        self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

root = Tk()
app = App(root, '')
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
